@@ -54,6 +54,7 @@ draw.gp <- function(object, type = 'rmse', log = FALSE, ...){
     seq_N <- 0
     wave_N <- length(object$design)
     if ( "type" %in% names(object$design) ) wave_N <- wave_N - 1
+    if ( "exclusion" %in% names(object$design) ) wave_N <- wave_N - 1
     if ( "x_test" %in% names(object$design) & "y_test" %in% names(object$design) ) wave_N <- wave_N - 2
     for ( i in 1:wave_N ){
       Ni <- sum(object$design[[paste('wave',i,sep='')]]$enrichment)
@@ -76,7 +77,8 @@ draw.gp <- function(object, type = 'rmse', log = FALSE, ...){
     seq_N <- 0
     wave_N <- length(object$design)
     if ( "type" %in% names(object$design) ) wave_N <- wave_N - 1
-    cust <- ifelse ( is.function(object$design$type), T, F )
+    if ( "exclusion" %in% names(object$design) ) wave_N <- wave_N - 1
+    cust <- object$design$type=='customized'
     if ( "x_test" %in% names(object$design) & "y_test" %in% names(object$design) ) wave_N <- wave_N - 2
     for ( i in 1:wave_N ){
       Ni <- sum(object$design[[paste('wave',i,sep='')]]$enrichment)
@@ -173,6 +175,7 @@ draw.dgp <- function(object, type = 'rmse', log = FALSE, ...){
     seq_N <- 0
     wave_N <- length(object$design)
     if ( "type" %in% names(object$design) ) wave_N <- wave_N - 1
+    if ( "exclusion" %in% names(object$design) ) wave_N <- wave_N - 1
     if ( "x_test" %in% names(object$design) & "y_test" %in% names(object$design) ) wave_N <- wave_N - 2
     for ( i in 1:wave_N ){
       Ni <- sum(object$design[[paste('wave',i,sep='')]]$enrichment)
@@ -196,7 +199,8 @@ draw.dgp <- function(object, type = 'rmse', log = FALSE, ...){
     seq_N <- 0
     wave_N <- length(object$design)
     if ( "type" %in% names(object$design) ) wave_N <- wave_N - 1
-    cust <- ifelse ( is.function(object$design$type), T, F )
+    if ( "exclusion" %in% names(object$design) ) wave_N <- wave_N - 1
+    cust <- object$design$type=='customized'
     if ( "x_test" %in% names(object$design) & "y_test" %in% names(object$design) ) wave_N <- wave_N - 2
     for ( i in 1:wave_N ){
       Ni <- sum(object$design[[paste('wave',i,sep='')]]$enrichment)
@@ -312,6 +316,7 @@ draw.bundle <- function(object, emulator = 1, type = 'rmse', log = FALSE, ...){
     seq_N <- 0
     wave_N <- length(object$design)
     if ( "type" %in% names(object$design) ) wave_N <- wave_N - 1
+    if ( "exclusion" %in% names(object$design) ) wave_N <- wave_N - 1
     if ( "x_test" %in% names(object$design) & "y_test" %in% names(object$design) ) wave_N <- wave_N - 2
     for ( i in 1:wave_N ){
       Ni <- sum(object$design[[paste('wave',i,sep='')]]$enrichment[,emulator])
@@ -336,7 +341,8 @@ draw.bundle <- function(object, emulator = 1, type = 'rmse', log = FALSE, ...){
     seq_N <- 0
     wave_N <- length(object$design)
     if ( "type" %in% names(object$design) ) wave_N <- wave_N - 1
-    cust <- ifelse ( is.function(object$design$type), T, F )
+    if ( "exclusion" %in% names(object$design) ) wave_N <- wave_N - 1
+    cust <- object$design$type=='customized'
     if ( "x_test" %in% names(object$design) & "y_test" %in% names(object$design) ) wave_N <- wave_N - 2
     for ( i in 1:wave_N ){
       Ni <- sum(object$design[[paste('wave',i,sep='')]]$enrichment[,emulator])
