@@ -8,7 +8,9 @@
 #' * the S3 class `dgp`.
 #' * the S3 class `bundle`.
 #' @param x_cand a matrix (with each row being a design point and column being an input dimension) that gives a candidate set
-#'     from which the next design point(s) are determined.
+#'     from which the next design point(s) are determined. If `object` is an instance of the `bundle` class, `x_cand` could also
+#'     be a list with the length equal to the number of emulators contained in the `object`. Each slot in `x_cand` is a matrix
+#'     that gives a candidate set for each emulator included in the bundle. See *Note* section below for further information.
 #' @param batch_size an integer that gives the number of design points to be chosen.
 #'     Defaults to `1`.
 #' @param workers the number of workers/cores to be used for the criterion calculation. If set to `NULL`,
@@ -44,6 +46,8 @@
 #' @note
 #' * The column order of the first argument of `aggregate` must be consistent with the order of emulator output dimensions (if `object` is an instance of the
 #'     `dgp` class), or the order of emulators placed in `object` if `object` is an instance of the `bundle` class;
+#' * If `x_cand` is supplied as a list when `object` is an instance of `bundle` class and a `aggregate` function is provided, the matrices in `x_cand` must have
+#'   common rows (i.e., the candidate sets of emulators in the bundle have common input locations) so the `aggregate` function can be applied.
 #' * Any R vector detected in `x_cand` will be treated as a column vector and automatically converted into a single-column
 #'   R matrix.
 #' @references
@@ -151,8 +155,10 @@ vigf.gp <- function(object, x_cand, batch_size = 1, workers = 1, ...) {
       idx_x_cand <- idx_x_cand0[-idx]
     }
   }
-  pkg.env$py_gc$collect()
-  gc(full=T)
+  if ( batch_size!=1 ){
+    pkg.env$py_gc$collect()
+    gc(full=T)
+  }
   return(idx)
 }
 
@@ -263,8 +269,10 @@ vigf.dgp <- function(object, x_cand, batch_size = 1, workers = 1, threading = FA
     }
     idx <- matrix(idx, nrow = batch_size, byrow = T)
   }
-  pkg.env$py_gc$collect()
-  gc(full=T)
+  if ( batch_size!=1 ){
+    pkg.env$py_gc$collect()
+    gc(full=T)
+  }
   return(idx)
 }
 
@@ -283,6 +291,7 @@ vigf.bundle <- function(object, x_cand, batch_size = 1, workers = 1, threading =
   n_emulators <- length(object)
   if ( "data" %in% names(object) ) n_emulators <- n_emulators - 1
   if ( "design" %in% names(object) ) n_emulators <- n_emulators - 1
+  if ( "id" %in% names(object) ) n_emulators <- n_emulators - 1
   training_input <- object$data$X
   training_output <- object$data$Y
   n_dim_X <- ncol(training_input[[1]])
@@ -298,18 +307,42 @@ vigf.bundle <- function(object, x_cand, batch_size = 1, workers = 1, threading =
     }
   }
   #check x_cand
-  if ( !is.matrix(x_cand)&!is.vector(x_cand) ) stop("'x_cand' must be a vector or a matrix.", call. = FALSE)
-  if ( is.vector(x_cand) ) x_cand <- as.matrix(x_cand)
-  if ( ncol(x_cand)!=n_dim_X ) stop("'x_cand' and the training input have different number of dimensions.", call. = FALSE)
+  if ( !is.list(x_cand) ){
+    if ( !is.matrix(x_cand) ) {
+      if ( !is.vector(x_cand) ) {
+        stop("'x_cand' must be a vector, a matrix, or a list.", call. = FALSE)
+      }
+    }
+  }
+
+  if ( is.list(x_cand) ){
+    if (length(x_cand) != n_emulators) stop("When 'x_cand' is a list, the number of elements in it should match the number of emulators in the bundle.", call. = FALSE)
+    for ( i in 1:n_emulators ){
+      if ( is.vector(x_cand[[i]]) ) x_cand[[i]] <- as.matrix(x_cand[[i]])
+      if ( ncol(x_cand[[i]])!=n_dim_X ) stop("Elements in 'x_cand' have different number of dimensions with the training input.", call. = FALSE)
+    }
+    islist <- TRUE
+  } else {
+    if ( is.vector(x_cand) ) x_cand <- as.matrix(x_cand)
+    if ( ncol(x_cand)!=n_dim_X ) stop("'x_cand' and the training input have different number of dimensions.", call. = FALSE)
+    islist <- FALSE
+  }
+
   #check core number
   if( !is.null(workers) ) {
     workers <- as.integer(workers)
     if ( workers < 1 ) stop("The worker number must be >= 1.", call. = FALSE)
   }
+
   #check aggregate
   if ( !is.null(aggregate) ){
     add_arg <- list(...)
     gnames <- methods::formalArgs(aggregate)
+    if (islist){
+      x_cand_dfs <- lapply(x_cand, as.data.frame)
+      x_cand <- as.matrix(Reduce(function(x, y) merge(x, y, all = FALSE), x_cand_dfs))
+      if (length(x_cand)==0) stop("Elements in 'x_cand' must have common positions when 'aggregate' is used.", call. = FALSE)
+    }
   }
   #check batch size
   batch_size <- as.integer(batch_size)
@@ -317,26 +350,27 @@ vigf.bundle <- function(object, x_cand, batch_size = 1, workers = 1, threading =
 
   #locate
   if ( batch_size==1 ){
-    scores <- c()
+    scores <- vector('list', n_emulators)
     for ( i in 1:n_emulators ){
       obj_i <- object[[paste('emulator',i,sep='')]]
       if ( inherits(obj_i,"gp") ){
-        res = obj_i$emulator_obj$metric(x_cand = x_cand, method = 'VIGF', score_only = TRUE)
-        scores <- cbind(scores, res)
+        res = obj_i$emulator_obj$metric(x_cand = if (is.list(x_cand)) {x_cand[[i]]} else {x_cand}, method = 'VIGF', score_only = TRUE)
+        scores[[i]] <- res
       } else {
         obj_i$emulator_obj$set_nb_parallel(threading)
         if ( identical(workers,as.integer(1)) ){
-          res = obj_i$emulator_obj$metric(x_cand = x_cand, method = 'VIGF', obj = obj_i$constructor_obj, score_only = TRUE)
+          res = obj_i$emulator_obj$metric(x_cand = if (is.list(x_cand)) {x_cand[[i]]} else {x_cand}, method = 'VIGF', obj = obj_i$constructor_obj, score_only = TRUE)
         } else {
-          res = obj_i$emulator_obj$pmetric(x_cand = x_cand, method = 'VIGF', obj = obj_i$constructor_obj, score_only = TRUE, core_num = workers)
+          res = obj_i$emulator_obj$pmetric(x_cand = if (is.list(x_cand)) {x_cand[[i]]} else {x_cand}, method = 'VIGF', obj = obj_i$constructor_obj, score_only = TRUE, core_num = workers)
         }
-        scores <- cbind(scores, res)
+        scores[[i]] <- if(ncol(res) == 1) res else rowMeans(res)
       }
     }
 
     if ( is.null(aggregate) ){
-      idx <- pkg.env$np$argmax(scores, axis=0L) + 1
+      idx <- sapply(scores, function(x){pkg.env$np$argmax(x, axis=0L) + 1})
     } else {
+      scores <- do.call(cbind, scores)
       if ("..." %in% gnames){
         agg_scores <- do.call(aggregate, c(list(scores), add_arg))
       } else {
@@ -345,13 +379,19 @@ vigf.bundle <- function(object, x_cand, batch_size = 1, workers = 1, threading =
         agg_scores <- do.call(aggregate, c(list(scores), gparam))
       }
       idx <- which.max(agg_scores)
-      idx <- rep(idx, n_emulators)
+      if (islist){
+        chosen_row <- x_cand[idx, ]
+        idx <- unlist(lapply(x_cand_dfs, function(df) {
+          which(colSums(t(as.matrix(df)) == chosen_row, na.rm = TRUE) == n_dim_X)
+        }))
+      } else {
+        idx <- rep(idx, n_emulators)
+      }
     }
-
     idx <- matrix(idx, nrow = 1, byrow = T)
   } else {
     idx <- c()
-    idx_x_cand0 <- c(1:nrow(x_cand))
+    idx_x_cand0 <- if (is.list(x_cand)) {lapply(x_cand, function(x){c(1:nrow(x))})} else replicate(n_emulators, c(1:nrow(x_cand)), simplify = FALSE)
     idx_x_cand <- idx_x_cand0
     constructor_obj_list <- list()
     emulator_obj_list <- list()
@@ -362,28 +402,28 @@ vigf.bundle <- function(object, x_cand, batch_size = 1, workers = 1, threading =
     }
 
     for (i in 1:batch_size){
-
-      scores <- c()
+      scores <- vector('list', n_emulators)
       for ( j in 1:n_emulators ){
         obj_j <- object[[paste('emulator',j,sep='')]]
         if ( inherits(obj_j,"gp") ){
-          res = emulator_obj_list[[j]]$metric(x_cand = x_cand[idx_x_cand,,drop=F], method = 'VIGF', score_only = TRUE)
-          scores <- cbind(scores, res)
+          res = emulator_obj_list[[j]]$metric(x_cand = if (is.list(x_cand)) {x_cand[[j]][idx_x_cand[[j]],,drop=F]} else {x_cand[idx_x_cand[[j]],,drop=F]}, method = 'VIGF', score_only = TRUE)
+          scores[[j]] <- res
         } else {
           emulator_obj_list[[j]]$set_nb_parallel(threading)
           if ( identical(workers,as.integer(1)) ){
-            res = emulator_obj_list[[j]]$metric(x_cand = x_cand[idx_x_cand,,drop=F], obj = emulator_obj_list[[j]]$constructor_obj, method = 'VIGF', score_only = TRUE)
+            res = emulator_obj_list[[j]]$metric(x_cand = if (is.list(x_cand)) {x_cand[[j]][idx_x_cand[[j]],,drop=F]} else {x_cand[idx_x_cand[[j]],,drop=F]}, obj = constructor_obj_list[[j]], method = 'VIGF', score_only = TRUE)
           } else {
-            res = emulator_obj_list[[j]]$pmetric(x_cand = x_cand[idx_x_cand,,drop=F], obj = emulator_obj_list[[j]]$constructor_obj, method = 'VIGF', score_only = TRUE, core_num = workers)
+            res = emulator_obj_list[[j]]$pmetric(x_cand = if (is.list(x_cand)) {x_cand[[j]][idx_x_cand[[j]],,drop=F]} else {x_cand[idx_x_cand[[j]],,drop=F]}, obj = constructor_obj_list[[j]], method = 'VIGF', score_only = TRUE, core_num = workers)
           }
           emulator_obj_list[[j]]$set_nb_parallel(FALSE)
-          scores <- cbind(scores, if(ncol(res) == 1) res else rowMeans(res))
+          scores[[j]] <- if(ncol(res) == 1) res else rowMeans(res)
         }
       }
 
       if ( is.null(aggregate) ){
-        idx_i <- pkg.env$np$argmax(scores, axis=0L) + 1
+        idx_i <- sapply(scores, function(x){pkg.env$np$argmax(x, axis=0L) + 1})
       } else {
+        scores <- do.call(cbind, scores)
         if ("..." %in% gnames){
           agg_scores <- do.call(aggregate, c(list(scores), add_arg))
         } else {
@@ -391,32 +431,48 @@ vigf.bundle <- function(object, x_cand, batch_size = 1, workers = 1, threading =
           gparam <- add_arg[gnames[gidx]]
           agg_scores <- do.call(aggregate, c(list(scores), gparam))
         }
+
         idx_i <- which.max(agg_scores)
         idx_i <- rep(idx_i, n_emulators)
       }
 
-      X_new <- x_cand[idx_x_cand,,drop=F][idx_i,,drop=F]
+      if (is.list(x_cand)){
+        X_new <- lapply(1:n_emulators, function(k) {x_cand[[k]][idx_x_cand[[k]],,drop=F][idx_i[k],,drop=F]})
+      } else {
+        X_new <- lapply(1:n_emulators, function(k) {x_cand[idx_x_cand[[k]],,drop=F][idx_i[k],,drop=F]})
+      }
 
       for ( j in 1:n_emulators ){
         obj_j <- object[[paste('emulator',j,sep='')]]
-        Y_new_j <- emulator_obj_list[[j]]$predict(X_new[j,,drop=F])[[1]]
+        Y_new_j <- emulator_obj_list[[j]]$predict(X_new[[j]])[[1]]
         if ( inherits(obj_j,"gp") ){
-          emulator_obj_list[[j]]$update_xy(rbind(training_input[[j]], X_new[j,]), rbind(training_output[[j]], Y_new_j))
+          emulator_obj_list[[j]]$update_xy(rbind(training_input[[j]], X_new[[j]]), rbind(training_output[[j]], Y_new_j))
         } else {
           B <- as.integer(length(obj_j$emulator_obj$all_layer_set))
           burnin <- obj_j$constructor_obj$burnin
           isblock <- obj_j$constructor_obj$block
-          constructor_obj_list[[j]]$update_xy(rbind(training_input[[j]], X_new[j,]), rbind(training_output[[j]], Y_new_j))
+          constructor_obj_list[[j]]$update_xy(rbind(training_input[[j]], X_new[[j]]), rbind(training_output[[j]], Y_new_j))
           est_obj <- constructor_obj_list[[j]]$estimate(burnin)
           emulator_obj_list[[j]] <- pkg.env$dgpsi$emulator(all_layer = est_obj, N = B, block = isblock)
         }
       }
-      idx <- c(idx,  idx_x_cand[idx_i])
-      idx_x_cand <- idx_x_cand0[-unique(idx)]
+
+      if ( !is.null(aggregate) & islist ){
+        chosen_row <- x_cand[idx_x_cand[[1]],,drop=F][idx_i[1], ]
+        idx <- rbind( idx, unlist(lapply(x_cand_dfs, function(df) {
+          which(apply(df, 1, function(row) identical(row, chosen_row)))
+        })))
+        idx_x_cand <- replicate(n_emulators, idx_x_cand[[1]][-idx_i[1]], simplify = FALSE)
+      } else {
+        idx <- rbind( idx,  sapply(1:n_emulators, function(k){idx_x_cand[[k]][idx_i[k]]}) )
+        idx_x_cand <- lapply(1:n_emulators, function(k){idx_x_cand0[[k]][-idx[,k]]})
+      }
     }
-    idx <- matrix(idx, nrow = batch_size, byrow = T)
+    #idx <- matrix(idx, nrow = batch_size, byrow = T)
+  }
+  if ( batch_size!=1 ){
+    pkg.env$py_gc$collect()
+    gc(full=T)
   }
-  pkg.env$py_gc$collect()
-  gc(full=T)
   return(idx)
 }
